#!/bin/bash
# This script creates an Aegir platform, from the download up to the front-end.

#****************************************
# General variables
#****************************************

# The name of this file.
kplatforms_this_file=$(basename $0)

# The full path of this file.
kplatforms_this_path=$(dirname $(pwd)/$0)

# Today's date and time.
kplatforms_date="$(date +%Y).$(date +%m).$(date +%d)"
kplatforms_time="$(date +%H).$(date +%M).$(date +%S)"

# Requirements
kplatforms_req_user="aegir"
kplatforms_req_drush_version="4.5"
kplatforms_req_drush_make_version="2.3"

# Warning detection.
kplatforms_warning=0

# Echos
echo_do="DO:"
echo_error="ERROR:"
echo_note="NOTE:"
echo_ok="OK:"
echo_warning="WARNING:"

#****************************************
# Usage
#****************************************

function kplatforms_echo_usage_and_exit {
  echo "Usage:"
  echo " $kplatforms_this_file [-a htpasswd_file] [-C | -c config_file] [-d drush_command] [-f] [-G] [-I | -i inclusions] [-M | -m additional_makefiles] [-n platform_name] [-o hostmaster_instance_name] [-p platforms_path] [-s platform_name_suffix] [-v drupal_version] [-w web_server]"
  echo "See the 'README.txt' file for complete explanation."
  exit 1
}

#****************************************
# Arguments
#****************************************

# Processing arguments and defining variables.
while getopts ":a:Cc:d:fGHhIi:Mm:n:o:p:s:v:w:" opt; do
  case $opt in

    a) # -a htpasswd_file
      kplatforms_htpasswd_file="$OPTARG"
      ;;

    C) # Do not read default configuration file.
      kplatforms_config_file="none"
      ;;

    c) # -c config_file
      kplatforms_config_file="$OPTARG"
      ;;

    d) # -d drush_command
      kplatforms_drush_command="$OPTARG"
      ;;

    f) # Using "drush --force-complete".
      kplatforms_force_complete="true"
      ;;

    G) # Do not import the platform into Aegir.
      kplatforms_import_into_aegir="false"
      ;;

    H|h) # Help
      kplatforms_echo_usage_and_exit
      ;;

    I) # No inclusions.
      kplatforms_inclusions="none"
      ;;

    i) # -i inclusions
      kplatforms_inclusions="$OPTARG"
      ;;

    M) # Do not include default additional makefile.
      kplatforms_additional_makefiles="none"
      ;;

    m) # -m /path/to/makefile.make
      kplatforms_additional_makefiles="$OPTARG"
      ;;

    n) # -n platform_name
      kplatforms_platform_name="$OPTARG"
      ;;

    o) # -m hostmaster_instance_name
      kplatforms_hostmaster_instance_name="$OPTARG"
      ;;

    p) # -p platforms_path
      kplatforms_platforms_path="$OPTARG"
      ;;

    s) # -s platform_name_suffix
      kplatforms_platform_name_suffix="$OPTARG"
      ;;

    v) # -v drupal_version
      kplatforms_drupal_version="$OPTARG"

      # Checking if Drupal version is supported.
      if [ $kplatforms_drupal_version != 8 -a $kplatforms_drupal_version != 7 -a $kplatforms_drupal_version != 6 -a $kplatforms_drupal_version != 5 ] ; then
        echo "$echo_error Version '$kplatforms_drupal_version' is not supported."
        echo "       Version must be '8', '7', '6' or '5'. Default is '7'."
        kplatforms_echo_usage_and_exit
      fi

      ;;

    w) # -w web_server
      kplatforms_web_server="$OPTARG"
      ;;

    \?) # Invalid option.
      echo "$echo_error Invalid option: -$OPTARG"
      kplatforms_echo_usage_and_exit
      ;;

    :) # Missing argument.
      echo "$echo_error Option -$OPTARG requires an argument."
      kplatforms_echo_usage_and_exit
      ;;

  esac
done

#****************************************
# Configuration file
#****************************************

function kplatforms_read_config_file {

  . $kplatforms_config_file

  if [ $? -eq 0 ]; then   # Test exit code.
    echo "$echo_ok Configuration file '$kplatforms_config_file' read successfully."
  else
    echo "$echo_error Cannot read configuration file '$kplatforms_config_file'."
    kplatforms_echo_usage_and_exit
  fi

}

if [ -z $kplatforms_config_file ]; then   # Reading the default configuration file if no one has been defined.

  kplatforms_config_file="/var/aegir/config/kplatforms.conf"

  if [ -r "$kplatforms_config_file" ]; then
    kplatforms_read_config_file
  fi

elif [ "$kplatforms_config_file" != "none" ]; then   # Processing the configuration file from argument.

  if [ -r "$kplatforms_config_file" ]; then   # Checking if the file is readable.
    kplatforms_read_config_file
  else
    echo "$echo_error The configuration file '$kplatforms_config_file' cannot be read."
    kplatforms_echo_usage_and_exit
  fi

fi

#****************************************
# Default variables
#****************************************

# Defining variables if not already defined neither by argument, config file nor shell variable.

# Drupal core major version number.
if [ -z $kplatforms_drupal_version ]; then
  kplatforms_drupal_version=7
fi
echo "$echo_note Variable kplatforms_drupal_version = '$kplatforms_drupal_version'"

# Platform name.
if [ -z $kplatforms_platform_name ]; then
  kplatforms_platform_name="drupal-$kplatforms_drupal_version"
fi
echo "$echo_note Variable kplatforms_platform_name = '$kplatforms_platform_name'"

# Platform name dated.
if [ -z $kplatforms_platform_name_dated ]; then
  kplatforms_platform_name_dated="true"
fi
echo "$echo_note Variable kplatforms_platform_name_dated = '$kplatforms_platform_name_dated'"

# Drush command.
if [ -z $kplatforms_drush_command ]; then
  kplatforms_drush_command="drush"
fi
echo "$echo_note Variable kplatforms_drush_command = '$kplatforms_drush_command'"

# File favicon.ico
if [ -z $kplatforms_favicon ]; then
  kplatforms_favicon="$kplatforms_this_path/favicon.ico"
fi
echo "$echo_note Variable kplatforms_favicon = '$kplatforms_favicon'"

# Hosmaster instance name.
# To use like "drush @hostmaster ...".
if [ -z $kplatforms_hostmaster_instance_name ]; then
  kplatforms_hostmaster_instance_name="hostmaster"
fi
echo "$echo_note Variable kplatforms_hostmaster_instance_name = '$kplatforms_hostmaster_instance_name'"

# Import the platform into Aegir.
if [ -z $kplatforms_import_into_aegir ]; then
  kplatforms_import_into_aegir="true"
fi
echo "$echo_note Variable kplatforms_import_into_aegir = '$kplatforms_import_into_aegir'"

# Inclusions
if [ -z $kplatforms_inclusions ]; then
  if [ $kplatforms_drupal_version = 8 ] ; then
    kplatforms_inclusions="none"
  else
    kplatforms_inclusions="modules themes"
  fi
fi
echo "$echo_note Variable kplatforms_inclusions = '$kplatforms_inclusions'"

# Platforms path, directory where to create platforms.
if [ -z $kplatforms_platforms_path ]; then
  kplatforms_platforms_path="/var/aegir/platforms"
fi
echo "$echo_note Variable kplatforms_platforms_path = '$kplatforms_platforms_path'"

# Temporary directory where to build the custom makefile.
if [ -z $kplatforms_tmpdir ]; then
  kplatforms_tmpdir="/tmp"
fi
echo "$echo_note Variable kplatforms_tmpdir = '$kplatforms_tmpdir'"

# Web server.
if [ -z $kplatforms_web_server ]; then
  kplatforms_web_server="master"
fi
echo "$echo_note Variable kplatforms_web_server = '$kplatforms_web_server'"

#****************************************
# Checking requirements
#****************************************

# User

kplatforms_user=$(id -nu)
echo "$echo_note Current user = '$kplatforms_user'"

if [ "$kplatforms_user" != "$kplatforms_req_user" ] ; then
  echo "$echo_error You must run '$kplatforms_this_file' as user '$kplatforms_req_user'."
  exit 1
fi

# Drush version

kplatforms_drush_version=$($kplatforms_drush_command --version | awk '{print $NF}')
echo "$echo_note Current 'drush' version = '$kplatforms_drush_version'"

if [ $(echo "$kplatforms_drush_version < $kplatforms_req_drush_version" | bc) -eq 1 ]; then
  echo "$echo_error 'Drush' version must be '$kplatforms_req_drush_version' or higher."
  exit 1
fi

# Drush make version

kplatforms_drush_make_version=$($kplatforms_drush_command make --version | awk '{print $NF}')
echo "$echo_note Current 'drush make' version = '$kplatforms_drush_make_version'"

if [ $(echo "$kplatforms_drush_make_version < $kplatforms_req_drush_make_version" | bc) -eq 1 ]; then
  echo "$echo_error 'Drush make' version must be '$kplatforms_req_drush_make_version' or higher."
  exit 1
fi

#****************************************
# Building platform information
#****************************************

# Do we write the creation date in the platform name ?
kplatforms_platform_name_dated=$(echo $kplatforms_platform_name_dated | tr '[:upper:]' '[:lower:]')   # Replacing upper case by lower case.
if [ "$kplatforms_platform_name_dated" = "true" ]; then
  kplatforms_platform_name_date="-$kplatforms_date"
else
  kplatforms_platform_name_date=""
fi

# Do we write a platform name suffix ?
if [ ! "$kplatforms_platform_name_suffix" = "" ]; then
  kplatforms_platform_name_suffix="-$kplatforms_platform_name_suffix"
fi

# The name of the platform directory without any path.
kplatforms_platform_directory="$kplatforms_platform_name$kplatforms_platform_name_date$kplatforms_platform_name_suffix"
echo "$echo_note Platform name = '$kplatforms_platform_directory'"

# Absolute path to the platform.
kplatforms_platform_path="$kplatforms_platforms_path/$kplatforms_platform_directory"
echo "$echo_note Platform path = '$kplatforms_platform_path'"

#****************************************
# Building custom makefile with inclusions
#****************************************

# Defining the directory where to find the makefiles to include.
kplatforms_includir="$kplatforms_this_path/includes/$kplatforms_drupal_version"
echo "$echo_note Includes directory = '$kplatforms_includir'"

# Defining the custom makefile with its full path.
kplatforms_custom_makefile="$kplatforms_tmpdir/kplatforms-$kplatforms_date.$kplatforms_time-$kplatforms_platform_directory.make"
echo "$echo_note Custom makefile = '$kplatforms_custom_makefile'"

# Replacing upper case by lower case.
kplatforms_inclusions=$(echo $kplatforms_inclusions | tr '[:upper:]' '[:lower:]')

# Replacing comas by space.
kplatforms_inclusions=$(echo $kplatforms_inclusions | sed 's/,/ /g')

echo "$echo_note Inclusions = '$kplatforms_inclusions'"

# Defining the core makefile.
kplatforms_core_makefile="$kplatforms_includir/core.make"

# Creating the custom makefile by including the core makefile.
echo "$echo_do Including makefile '$kplatforms_core_makefile'"
cat $kplatforms_core_makefile >$kplatforms_custom_makefile

# Check if no inclusions is defined.
if [ "$kplatforms_inclusions" != "none" ]; then

  # Processing inclusions one by one.
  for kplatforms_inclusion in $kplatforms_inclusions; do

    # The makefile with its full path.
    kplatforms_inclusion_makefile="$kplatforms_includir/$kplatforms_inclusion.make"

    # Checking if we can read the makefile.
    if [ ! -r "$kplatforms_inclusion_makefile" ]; then
      echo "$echo_error The makefile '$kplatforms_inclusion_makefile' cannot be read."
      rm $kplatforms_custom_makefile
      kplatforms_echo_usage_and_exit
    fi

    # Including the makefile.
    echo "$echo_do Including makefile '$kplatforms_inclusion_makefile'"
    cat $kplatforms_inclusion_makefile >>$kplatforms_custom_makefile

  done

fi

#****************************************
# Reducing items in the custom makefile
#****************************************

# This reduces the list of libraries in the platform if a list is given.

if [ ! -z $kplatforms_unwanted_libraries ]; then   # If a list of unwanted libraries is defined...

  # Replacing upper case by lower case.
  kplatforms_unwanted_libraries=$(echo $kplatforms_unwanted_libraries | tr '[:upper:]' '[:lower:]')

  # Replacing comas by space.
  kplatforms_unwanted_libraries=$(echo $kplatforms_unwanted_libraries | sed 's/,/ /g')

  echo "$echo_note Unwanted libraries = '$kplatforms_unwanted_libraries'"

  # Disabling specified libraries in the custom makefile.
  for kplatforms_unwanted_library in $kplatforms_unwanted_libraries; do
    echo "$echo_do Disabling $kplatforms_unwanted_library."
    sed -i "/^libraries\[$kplatforms_unwanted_library\]/ s/^/;/" $kplatforms_custom_makefile
  done

elif [ ! -z $kplatforms_wanted_libraries ]; then   # If a list of wanted libraries is defined...

  # Replacing upper case by lower case.
  kplatforms_wanted_libraries=$(echo $kplatforms_wanted_libraries | tr '[:upper:]' '[:lower:]')

  # Replacing comas by space.
  kplatforms_wanted_libraries=$(echo $kplatforms_wanted_libraries | sed 's/,/ /g')

  echo "$echo_note Wanted libraries = '$kplatforms_wanted_libraries'"

  echo "$echo_do Disabling all libraries in the custom makefile."
  sed -i "/^libraries/ s/^/;/" $kplatforms_custom_makefile

  # Enabling specified libraries in the custom makefile.
  for kplatforms_wanted_library in $kplatforms_wanted_libraries; do
    echo "$echo_do Enabling $kplatforms_wanted_library."
    sed -i "/^;libraries\[$kplatforms_wanted_library\]/ s/^;//" $kplatforms_custom_makefile
  done

fi

# This reduces the list of projects in the platform if a list is given.

if [ ! -z $kplatforms_unwanted_projects ]; then   # If a list of unwanted projects is defined...

  # Replacing upper case by lower case.
  kplatforms_unwanted_projects=$(echo $kplatforms_unwanted_projects | tr '[:upper:]' '[:lower:]')

  # Replacing comas by space.
  kplatforms_unwanted_projects=$(echo $kplatforms_unwanted_projects | sed 's/,/ /g')

  echo "$echo_note Unwanted projects = '$kplatforms_unwanted_projects'"

  # Disabling specified projects in the custom makefile.
  for kplatforms_unwanted_project in $kplatforms_unwanted_projects; do
    echo "$echo_do Disabling $kplatforms_unwanted_project."
    sed -i "/^projects\[$kplatforms_unwanted_project\]/ s/^/;/" $kplatforms_custom_makefile
  done

elif [ ! -z $kplatforms_wanted_projects ]; then   # If a list of wanted projects is defined...

  # Replacing upper case by lower case.
  kplatforms_wanted_projects=$(echo $kplatforms_wanted_projects | tr '[:upper:]' '[:lower:]')

  # Replacing comas by space.
  kplatforms_wanted_projects=$(echo $kplatforms_wanted_projects | sed 's/,/ /g')

  echo "$echo_note Wanted projects = '$kplatforms_wanted_projects'"

  echo "$echo_do Disabling all projects in the custom makefile."
  sed -i "/^projects/ s/^/;/" $kplatforms_custom_makefile

  echo "$echo_do Enabling Drupal core."
  sed -i "/^;projects\[drupal\]\|^;projects\[pressflow\]/ s/^;//" $kplatforms_custom_makefile

  # Enabling specified projects in the custom makefile.
  for kplatforms_wanted_project in $kplatforms_wanted_projects; do
    echo "$echo_do Enabling $kplatforms_wanted_project."
    sed -i "/^;projects\[$kplatforms_wanted_project\]/ s/^;//" $kplatforms_custom_makefile
  done

fi

#****************************************
# Adding additional makefiles
#****************************************

function kplatforms_include_additional_makefiles {

  # Replacing comas by space.
  kplatforms_additional_makefiles=$(echo $kplatforms_additional_makefiles | sed 's/,/ /g')

  # Processing makefiles one by one.
  for kplatforms_additional_makefile in $kplatforms_additional_makefiles; do

    if [ -r "$kplatforms_additional_makefile" ]; then

      echo "$echo_do Adding makefile '$kplatforms_additional_makefile'"
      cat $kplatforms_additional_makefile >>$kplatforms_custom_makefile

      if [ $? -ne 0 ]; then   # Test exit code.
        echo "$echo_error Cannot include makefile '$kplatforms_additional_makefile'."
        kplatforms_echo_usage_and_exit
      fi

    else
      echo "$echo_error The makefile '$kplatforms_additional_makefile' cannot be read."
      rm $kplatforms_custom_makefile
      kplatforms_echo_usage_and_exit
    fi

  done

}

if [ -z $kplatforms_additional_makefiles ]; then   # Including default additional makefile if no one has been defined.
  kplatforms_additional_makefiles="/var/aegir/makefiles/kplatforms.make"

  if [ -r "$kplatforms_additional_makefiles" ]; then
    kplatforms_include_additional_makefiles
  fi

elif [ "$kplatforms_additional_makefiles" != "none" ]; then   # Including makefiles from argument.
  kplatforms_include_additional_makefiles
fi

#****************************************
# Drush commands
#****************************************

# Are we using "drush --force-complete" ?
kplatforms_force_complete=$(echo $kplatforms_force_complete | tr '[:upper:]' '[:lower:]')   # Replacing upper case by lower case.
if [ "$kplatforms_force_complete" = "true" ]; then
  kplatforms_drush_command="$kplatforms_drush_command --force-complete"
fi
echo "$echo_note Drush command = '$kplatforms_drush_command'"

# Downloading the platform.
echo "$echo_do Downloading the platform..."
$kplatforms_drush_command make $kplatforms_custom_makefile $kplatforms_platform_path

# Test exit code.
if [ $? -eq 0 ]; then
  echo "$echo_ok Platform downloaded successfully in '$kplatforms_platform_path'."
else
  echo "$echo_error Platform not downloaded successfully."
  exit 1
fi

# If we import the platform into Aegir.
if [ "$kplatforms_import_into_aegir" = "true" ]; then

  # TODO: Remove these lines when Aegir supports Drupal 8.
  if [ $kplatforms_drupal_version = 8 ] ; then
    echo "$echo_warning Aegir / Provision does not yet support Drupal 8 platforms."
    echo "         See https://drupal.org/node/1194602"
    echo "         and https://drupal.org/node/1892496 for more details."
    kplatforms_warning=`expr $kplatforms_warning + 1`
  fi

  # Saving Drush alias.
  echo "$echo_do Saving Drush alias."
  $kplatforms_drush_command provision-save @platform_$kplatforms_platform_directory \
    --context_type=platform \
    --web_server=@server_$kplatforms_web_server \
    --root=$kplatforms_platform_path \
    --makefile=$kplatforms_custom_makefile

  # Test exit code and run the next command.
  if [ $? -eq 0 ]; then

    # Importing the platform into Aegir.
    echo "$echo_do Importing the platform into Aegir."
    $kplatforms_drush_command @$kplatforms_hostmaster_instance_name hosting-import @platform_$kplatforms_platform_directory

    # Test exit code.
    if [ $? -ne 0 ]; then
      echo "$echo_warning Platform not imported successfully."
      kplatforms_warning=`expr $kplatforms_warning + 1`
    fi

  else
    echo "$echo_warning Drush alias not saved successfully and platform not imported into Aegir."
    kplatforms_warning=`expr $kplatforms_warning + 1`
  fi

else
  echo "$echo_note Not importing the platform into Aegir."

fi

#****************************************
# Access restricted by users (htauth)
#****************************************

if [ "$kplatforms_htpasswd_file" != "" ]; then

  # Checking if we can read the htpasswd file.
  if [ ! -r "$kplatforms_htpasswd_file" ]; then
    echo "$echo_warning The htpasswd file cannot be read."
    echo "Use: 'htpasswd -b -c $kplatforms_htpasswd_file username password'."
    kplatforms_warning=`expr $kplatforms_warning + 1`
  fi

  # Defining variables.
  kplatforms_htaccess_file="$kplatforms_platform_path/.htaccess"
  kplatforms_htaccess_file_head="$kplatforms_platform_path/.htaccess.head"
  kplatforms_htaccess_file_tmp="$kplatforms_platform_path/.htaccess.tmp"

  # Editing htaccess file.
  echo "# Access restricted by users" >$kplatforms_htaccess_file_head && \
    echo "AuthUserFile $kplatforms_htpasswd_file" >>$kplatforms_htaccess_file_head && \
    echo "AuthName 'Authorized people only'" >>$kplatforms_htaccess_file_head && \
    echo "AuthType Basic" >>$kplatforms_htaccess_file_head && \
    echo "require valid-user" >>$kplatforms_htaccess_file_head && \
    echo "" >>$kplatforms_htaccess_file_head && \
    cat $kplatforms_htaccess_file_head $kplatforms_htaccess_file >$kplatforms_htaccess_file_tmp && \
    mv $kplatforms_htaccess_file_tmp $kplatforms_htaccess_file && \
    rm $kplatforms_htaccess_file_head

  # Test exit code.
  if [ $? -eq 0 ]; then
    echo "$echo_ok Sites on this platform restricted successfully with '$kplatforms_htpasswd_file'."
  else
    echo "$echo_warning Sites on this platform not restricted successfully."
    kplatforms_warning=`expr $kplatforms_warning + 1`
  fi

fi

#****************************************
# File favicon.ico
#****************************************

# Checking if we can read the file.
if [ -r "$kplatforms_favicon" ]; then

  # Copying the file into the platform.
  cp $kplatforms_favicon $kplatforms_platform_path/favicon.ico

  # Test exit code.
  if [ $? -eq 0 ]; then
    echo "$echo_ok File favicon.ico copied successfully."
  else
    echo "$echo_warning File favicon.ico not copied successfully."
    kplatforms_warning=`expr $kplatforms_warning + 1`
  fi

else
  echo "$echo_warning The file '$kplatforms_favicon' cannot be read."
  kplatforms_warning=`expr $kplatforms_warning + 1`
fi

#****************************************
# TCPDF library
#****************************************

# TCPDF library for the Drupal 6 contrib module "Printer, e-mail and PDF versions" (print).
# Directories non-writable permissions.

# "TCPDF directory Non-writable permissions" (module print) out of the script
# http://drupal.org/node/1810716

if [ $kplatforms_drupal_version -eq 6 -a -d "$kplatforms_platform_path/sites/all/libraries/tcpdf" ]; then

  # Web server group:
  # Use www-data for Debian and Ubuntu.
  # Use apache for RedHat and CentOS.
  kplatforms_webserver_grp="www-data"

  # TODO add test for RedHat
  if [ -x /usr/bin/lsb_release ]; then
    kplatforms_os=$(lsb_release -i | cut -f 2)
    if [ "$kplatforms_os" = "CentOS" ]; then
      kplatforms_webserver_grp=apache
    fi
  fi

  # Modifying permissions and group.
  chmod g+w $kplatforms_platform_path/sites/all/libraries/tcpdf/cache $kplatforms_platform_path/sites/all/libraries/tcpdf/images && \
    chgrp $kplatforms_webserver_grp $kplatforms_platform_path/sites/all/libraries/tcpdf/cache $kplatforms_platform_path/sites/all/libraries/tcpdf/images

  # Test exit code.
  if [ $? -eq 0 ]; then
    echo "$echo_ok TCPDF chmod and chgrp."
  else
    echo "$echo_warning TCPDF chmod and chgrp not done successfully."
    kplatforms_warning=`expr $kplatforms_warning + 1`
  fi

fi

#****************************************
# Exiting
#****************************************

if [ $kplatforms_warning -eq 0 ]; then
  echo "$echo_ok '$kplatforms_this_file' terminated successfully."
  exit 0
elif [ $kplatforms_warning -eq 1 ]; then
  echo "$echo_warning '$kplatforms_this_file' terminated with $kplatforms_warning warning."
  exit $kplatforms_warning
else
  echo "$echo_warning '$kplatforms_this_file' terminated with $kplatforms_warning warnings."
  exit $kplatforms_warning
fi

#****************************************
# End
#****************************************
